

#ifndef __ATLAPP_H__
	#error DxApp.h requires atlapp.h to be included first
#endif


#include <vector>
#include "DxDef.h"


D3DFORMAT EnumerationFormats[] = {D3DFMT_X8R8G8B8, D3DFMT_A8R8G8B8, D3DFMT_A2R10G10B10, D3DFMT_X1R5G5B5,
		                              D3DFMT_A1R5G5B5, D3DFMT_R5G6B5};


//---------------------------------------------------------------------------------------
// Class:  CDxAdapter
// Desc:   Stores adaptor info.
//---------------------------------------------------------------------------------------

class CDxAdapter
{
public:

//---------------------------------------------------------------------------------------
// Constructor:  CDxAdapter(CComPtr<IDIRECT3D> d3d,UINT adapter)
// Desc:         Get's the adaptor's identifier, caps, and modes.
//---------------------------------------------------------------------------------------

  CDxAdapter(CComPtr<IDIRECT3D> d3d,UINT adapter)
  {
    D3DDISPLAYMODE mode;
    HRESULT        hr;

    ordinal = adapter;
    hr = d3d->GetAdapterIdentifier(ordinal,0,&adapterid);
    if(FAILED(hr))
    {
      throw hr;
    }

    hr = d3d->GetDeviceCaps(ordinal,D3DDEVTYPE_HAL,&caps);
    if(FAILED(hr))
    {
      throw hr;
    }

    for(UINT currentformat=0; currentformat < (sizeof(EnumerationFormats) / sizeof(D3DFORMAT)); currentformat++)
    {
      for(UINT i=0;i < d3d->GetAdapterModeCount(ordinal,EnumerationFormats[currentformat]);i++)
      {
        
        hr = d3d->EnumAdapterModes(ordinal,EnumerationFormats[currentformat],i,&mode);
        if(FAILED(hr))
        {
          throw hr;
        }

        modes.push_back(mode);
      }
    }
  }


//---------------------------------------------------------------------------------------
// Function:  GetAdapterModeCount
// Desc:      Returns the number of modes the adaptor supports.
//---------------------------------------------------------------------------------------

  UINT GetAdapterModeCount()
  {
    return modes.size();
  }


  UINT                        ordinal;
  D3DADAPTER_IDENTIFIER       adapterid;
  D3DCAPS                     caps;
  std::vector<D3DDISPLAYMODE> modes;

private:
  CDxAdapter(){}
};



//---------------------------------------------------------------------------------------
// Class:  CDxAdapterList
// Desc:   Stores a CDxAdapter class for each adaptor installed.
//---------------------------------------------------------------------------------------

class CDxAdapterList
{
public:


//---------------------------------------------------------------------------------------
// Function:  Init
// Desc:      Creates a CDxAdaptor for each installed adaptor and adds it to the vector.
//---------------------------------------------------------------------------------------

  HRESULT Init(CComPtr<IDIRECT3D> d3d)
  {
    //  Empty the list (if already initialized)
    if(adapters.size() > 0)
      Term();

    //  Add the adaptors
    try
    {
      for (UINT i=0;i < d3d->GetAdapterCount();i++)
      {
        adapters.push_back(CDxAdapter(d3d,i));
      }
    }
    catch(HRESULT hr)
    {
      return hr;
    }
    catch(...)
    {
      return E_FAIL;
    }

    return S_OK;
  }


//---------------------------------------------------------------------------------------
// Function:  Term
// Desc:      Clears the vector.
//---------------------------------------------------------------------------------------

  void Term()
  {
    adapters.erase(adapters.begin(),adapters.end());
  }


//---------------------------------------------------------------------------------------
// Function:  operator[]
// Desc:      Returns the CDxAdaptor with the given ordinal (i).
//---------------------------------------------------------------------------------------

  CDxAdapter& operator[](UINT i)
  {
    return adapters[i];
  }


//---------------------------------------------------------------------------------------
// Function:  GetAdapterCount
// Desc:      Returns the number of installed adaptors.
//---------------------------------------------------------------------------------------

  UINT GetAdapterCount()
  {
    return adapters.size();
  }

private:

  std::vector<CDxAdapter> adapters;
};



//---------------------------------------------------------------------------------------
// Class:  CDxAppModule
// Desc:   Module class for DirectX applications.
//---------------------------------------------------------------------------------------

class CDxAppModule : public CAppModule
{
public:


//---------------------------------------------------------------------------------------
// Function:  Init
// Desc:      Calls CAppModule::Init() and creates the Direct3D object.
//---------------------------------------------------------------------------------------

	HRESULT Init(_ATL_OBJMAP_ENTRY* pObjMap, HINSTANCE hInstance, const GUID* pLibID = NULL)
	{
		HRESULT hRet;
    
    hRet = CAppModule::Init(pObjMap, hInstance, pLibID);
		if(FAILED(hRet))
			return hRet;

    d3d = DIRECT3DCREATE(D3D_SDK_VERSION);
    if(!d3d)
      return E_FAIL;

    hRet = adaptors.Init(d3d);

    return hRet;
  }


//---------------------------------------------------------------------------------------
// Function:  Term
// Desc:      Calls CAppModule::Term() and releases the Direct3D object.
//---------------------------------------------------------------------------------------

  void Term()
  {
    adaptors.Term();
    d3d.Detach()->Release();

    CAppModule::Term();
  }


//---------------------------------------------------------------------------------------
// Function:  GetDirect3DInterface
// Desc:      Returns a reference to the Direct3D object.
//---------------------------------------------------------------------------------------

  CComPtr<IDIRECT3D> GetDirect3DInterface()
  {
    return d3d;
  }

  CDxAdapterList      adaptors;

private:
  CComPtr<IDIRECT3D> d3d;
};